package com.ocpsoft.pretty.component.renderer;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ocpsoft.pretty.component.Link;
import com.ocpsoft.pretty.component.PrettyUrlBuilder;

public class LinkRenderer extends Renderer
{
    public static final String RENDERER_TYPE = "javax.faces.Link";
    private final PrettyUrlBuilder urlBuilder = new PrettyUrlBuilder();

    @Override
    public void encodeBegin(final FacesContext context, final UIComponent component) throws IOException
    {
        super.encodeBegin(context, component);
        if (!component.isRendered())
        {
            return;
        }

        Link link = (Link) component;
        ResponseWriter writer = context.getResponseWriter();

        writer.startElement("a", link);

        String href = context.getExternalContext().getRequestContextPath() + urlBuilder.buildMappedUrl(component);
        writer.writeURIAttribute("href", context.getExternalContext().encodeResourceURL(href), "href");

        writer.writeAttribute("id", link.getClientId(context), "id");
        writer.writeAttribute("accesskey", link.getAccesskey(), "accesskey");
        writer.writeAttribute("charset", link.getCharset(), "charset");
        writer.writeAttribute("coords", link.getCoords(), "coords");
        writer.writeAttribute("dir", link.getDir(), "dir");
        writer.writeAttribute("hreflang", link.getHreflang(), "hreflang");
        writer.writeAttribute("lang", link.getLang(), "lang");
        writer.writeAttribute("onblur", link.getOnblur(), "onblur");
        writer.writeAttribute("onclick", link.getOnclick(), "onclick");
        writer.writeAttribute("ondblclick", link.getOndblclick(), "ondblclick");
        writer.writeAttribute("onfocus", link.getOnfocus(), "onfocus");
        writer.writeAttribute("onkeydown", link.getOnkeydown(), "onkeydown");
        writer.writeAttribute("onkeypress", link.getOnkeypress(), "onkeypress");
        writer.writeAttribute("onkeyup", link.getOnkeyup(), "onkeyup");
        writer.writeAttribute("onmousedown", link.getOnmousedown(), "onmousedown");
        writer.writeAttribute("onmousemove", link.getOnmousemove(), "onmousemove");
        writer.writeAttribute("onmouseout", link.getOnmouseout(), "onmouseout");
        writer.writeAttribute("onmouseover", link.getOnmouseover(), "onmouseover");
        writer.writeAttribute("onmouseup", link.getOnmouseup(), "onmouseup");
        writer.writeAttribute("rel", link.getRel(), "rel");
        writer.writeAttribute("rev", link.getRev(), "rev");
        writer.writeAttribute("shape", link.getShape(), "shape");
        writer.writeAttribute("style", link.getStyle(), "style");
        writer.writeAttribute("class", link.getStyleClass(), "styleClass");
        writer.writeAttribute("tabindex", link.getTabindex(), "tabindex");
        writer.writeAttribute("target", link.getTarget(), "target");
        writer.writeAttribute("title", link.getTitle(), "title");
        writer.writeAttribute("type", link.getType(), "type");
    }

    @Override
    public void encodeEnd(final FacesContext context, final UIComponent component) throws IOException
    {

        super.encodeEnd(context, component);

        if (!component.isRendered())
        {
            return;
        }

        ResponseWriter writer = context.getResponseWriter();
        writer.endElement("a");
    }
}